#AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"extract_tag": "om.ipynb",
         "contains_tag": "om.ipynb",
         "is_nbx": "om.ipynb",
         "is_nbx_cell": "om.ipynb",
         "is_magic_or_shell": "om.ipynb",
         "": "om.ipynb",
         "strip": "om.ipynb",
         "parse_xarg": "om.ipynb",
         "get_imports_from_src": "om.ipynb",
         "Import": "om.ipynb",
         "create_import_statement": "om.ipynb",
         "extract_imports_from": "om.ipynb",
         "Bunch": "om.ipynb",
         "load_nb": "om.ipynb",
         "parse_src": "om.ipynb",
         "parse_nbx_cell": "om.ipynb",
         "concat": "om.ipynb",
         "unzip": "om.ipynb",
         "negate": "om.ipynb",
         "is_constarg": "om.ipynb",
         "get_item": "om.ipynb",
         "get_items": "om.ipynb",
         "not_constarg": "om.ipynb",
         "parse_nb": "om.ipynb",
         "get_arrays": "om.ipynb",
         "init_job": "om.ipynb",
         "cont_job": "om.ipynb",
         "chain_jobs": "om.ipynb",
         "check_parsed_nb": "om.ipynb",
         "NbxBundle": "om.ipynb",
         "BUNDLE_SUMMARY": "om.ipynb",
         "ParameterSpace": "pspace.ipynb",
         "get_templ_args": "templ.ipynb",
         "render_templ": "templ.ipynb",
         "create_file_from_template": "templ.ipynb",
         "render_template_from_string": "templ.ipynb"}

modules = ["om.py",
           "pspace.py",
           "templ.py"]

git_url = "https://github.com/mirkoklukas/nbx/tree/master/"

def custom_doc_links(name): return None